# -*- coding: utf-8 -*-
##############################################################################
#
#    POS Membership module for OpenERP, Manage membership payments from POS.
#    Copyright (C) 2013 L'Heureux Cyclage (<http://www.heureux-cyclage.org>)
#
#    This file is a part of POS Membership
#
#    POS Membership is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    POS Membership is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp import netsvc
from openerp.osv import fields, osv
from openerp.tools.translate import _
import time
from datetime import datetime, date
from dateutil.relativedelta import relativedelta

STATE = [
    ('none', 'Non Member'),
    ('canceled', 'Cancelled Member'),
    ('old', 'Old Member'),
    ('waiting', 'Waiting Member'),
    ('invoiced', 'Invoiced Member'),
    ('free', 'Free Member'),
    ('paid', 'Paid Member'),
]


class pos_session(osv.osv):
    _inherit = 'pos.session'

    def open_cb(self, cr, uid, ids, context=None):
        """
        Avoid call the Point Of Sale interface and set the pos.session to 'opened' (in progress)
        """
        if context is None:
            context = dict()

        if isinstance(ids, (int, long)):
            ids = [ids]

        this_record = self.browse(cr, uid, ids[0], context=context)
        this_record._workflow_signal('open')

        context.update(active_id=this_record.id)

        return {
        }
pos_session()



class pos_order(osv.osv):
    _inherit = 'pos.order'

    def create_partner_from_ui(self, cr, uid, partners, context=None):
        partner_ids = []
        partner_obj = self.pool.get('res.partner')
        for tmp_partner in partners:
            partner = tmp_partner['data']
            partner_id = partner_obj.create(cr, uid, {
                'name': partner['name'],
            }, context)
            partner_ids.append(partner_id)
            #self.signal_paid(cr, uid, [partner_id])
        return partner_ids

    def create_from_ui(self, cr, uid, orders, context=None):
        # NOTE: copied from openerp/addons/point_of_sale/point_of_sale.py
        #_logger.info("orders: %r", orders)
        order_ids = []
        for tmp_order in orders:
            order = tmp_order['data']
            order_id = self.create(cr, uid, {
                'name': order['name'],
                'user_id': order['user_id'] or False,
                'session_id': order['pos_session_id'],
                'lines': order['lines'],
                'pos_reference': order['name'],
                #-- BEGIN pos_membership
                'partner_id': order['partner_id'],
                #-- END pos_membership
            }, context)

            for payments in order['statement_ids']:
                payment = payments[2]
                self.add_payment(cr, uid, order_id, {
                    'amount': payment['amount'] or 0.0,
                    'payment_date': payment['name'],
                    'statement_id': payment['statement_id'],
                    'payment_name': payment.get('note', False),
                    'journal': payment['journal_id']
                }, context=context)

            if order['amount_return']:
                session = self.pool.get('pos.session').browse(cr, uid, order['pos_session_id'], context=context)
                cash_journal = session.cash_journal_id
                cash_statement = False
                if not cash_journal:
                    cash_journal_ids = filter(lambda st: st.journal_id.type == 'cash', session.statement_ids)
                    if not len(cash_journal_ids):
                        raise osv.except_osv(_('error!'),
                                             _("No cash statement found for this session. Unable to record returned cash."))
                    cash_journal = cash_journal_ids[0].journal_id
                self.add_payment(cr, uid, order_id, {
                    'amount': -order['amount_return'],
                    'payment_date': time.strftime('%Y-%m-%d %H:%M:%S'),
                    'payment_name': _('return'),
                    'journal': cash_journal.id,
                }, context=context)
            order_ids.append(order_id)
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'pos.order', order_id, 'paid', cr)
        return order_ids

pos_order()


class membership_line(osv.osv):
    '''Member line'''
    _inherit = 'membership.membership_line'

    def _get_partners(self, cr, uid, ids, context=None):
        list_membership_line = []
        member_line_obj = self.pool.get('membership.membership_line')
        for partner in self.pool.get('res.partner').browse(cr, uid, ids, context=context):
            if partner.member_lines:
                list_membership_line += member_line_obj.search(cr, uid, [('id', 'in', [l.id for l in partner.member_lines])], context=context)
        return list_membership_line

    #def _get_membership_lines_from_account_invoice(self, cr, uid, ids, context=None):
    #    list_membership_line = []
    #    member_line_obj = self.pool.get('membership.membership_line')
    #    for invoice in self.pool.get('account.invoice').browse(cr, uid, ids, context=context):
    #        if invoice.invoice_line:
    #            list_membership_line += member_line_obj.search(cr, uid, [('account_invoice_line', 'in', [ l.id for l in invoice.invoice_line])], context=context)
    #    res = \
    #     { 'class': 'account.invoice'
    #     , 'ids': list_membership_line
    #     }
    #    #res= list_membership_line
    #    print("DEV: [pos_membership] [membership_line] [_get_membership_lines_from_account_invoice]: res=%s" % str(res))
    #    return res
    def _get_membership_lines_from_pos_order(self, cr, uid, ids, context=None):
        list_membership_line = []
        member_line_obj = self.pool.get('membership.membership_line')
        for order in self.pool.get('pos.order').browse(cr, uid, ids, context=context):
            if order.lines:
                list_membership_line += member_line_obj.search(cr, uid, [('pos_order_line', 'in', [l.id for l in order.lines])], context=context)
        #res = \
        # { 'class': 'pos.order'
        # , 'ids': list_membership_line
        # }
        res = list_membership_line
        return res

    def _check_membership_date(self, cr, uid, ids, context=None):
        """Check if membership product is not in the past
        @param self: The object pointer
        @param cr: the current row, from the database cursor,
        @param uid: the current user’s ID for security checks,
        @param ids: List of Membership Line IDs
        @param context: A standard dictionary for contextual values
        """

        cr.execute('''
         SELECT MIN(ml.date_to - ai.date_invoice)
             FROM membership_membership_line ml
             JOIN account_invoice_line ail ON (
                ml.account_invoice_line = ail.id
                )
            JOIN account_invoice ai ON (
            ai.id = ail.invoice_id)
            WHERE ml.id IN %s''', (tuple(ids),))
        res = cr.fetchall()
        for r in res:
            if r[0] and r[0] < 0:
                return False
        cr.execute('''
         SELECT MIN(ml.date_to - ai.date_order)
             FROM membership_membership_line ml
             JOIN pos_order_line ail ON (
                ml.pos_order_line = ail.id
                )
            JOIN pos_order ai ON (
            ai.id = ail.order_id)
            WHERE ml.id IN %s''', (tuple(ids),))
        res = cr.fetchall()
        for r in res:
            if r[0] and r[0] < 0:
                return False
        return True

    def _state(self, cr, uid, ids, name, args, context=None):
        """Compute the state lines
        @param self: The object pointer
        @param cr: the current row, from the database cursor,
        @param uid: the current user’s ID for security checks,
        @param ids: List of Membership Line IDs
        @param name: Field Name
        @param context: A standard dictionary for contextual values
        @param return: Dictionary of state Value
        """
        res = {}
        inv_obj = self.pool.get('account.invoice')
        ord_obj = self.pool.get('pos.order')
        for line in self.browse(cr, uid, ids, context=context):
            cr.execute('''
            SELECT i.state, i.id FROM
            account_invoice i
            WHERE
            i.id = (
                SELECT l.invoice_id FROM
                account_invoice_line l WHERE
                l.id = (
                    SELECT  ml.account_invoice_line FROM
                    membership_membership_line ml WHERE
                    ml.id = %s
                    )
                )
            ''', (line.id,))
            fetched = cr.fetchone()
            if fetched:
                istate = fetched[0]
                state = 'none'
                if (istate == 'draft') | (istate == 'proforma'):
                    state = 'waiting'
                elif istate == 'open':
                    state = 'invoiced'
                elif istate == 'paid':
                    state = 'paid'
                    inv = inv_obj.browse(cr, uid, fetched[1], context=context)
                    for payment in inv.payment_ids:
                        if payment.invoice and payment.invoice.type == 'out_refund':
                            state = 'canceled'
                elif istate == 'cancel':
                    state = 'canceled'
                res[line.id] = state
            else:
                cr.execute('''
                SELECT i.state, i.id FROM
                pos_order i
                WHERE
                i.id = (
                    SELECT l.order_id FROM
                    pos_order_line l WHERE
                    l.id = (
                        SELECT  ml.pos_order_line FROM
                        membership_membership_line ml WHERE
                        ml.id = %s
                        )
                    )
                ''', (line.id,))
                fetched = cr.fetchone()
                if not fetched:
                    res[line.id] = 'canceled'
                    continue
                self.write(cr, uid, line.id, {'partner': line.partner.id})
                # NOTE: force une mise à jour du partner,
                #       car il n'y en a pas lors de la création du pos.order
                #       dans le PoS (pour le moment)
                partner_obj = self.pool.get('res.partner')
                for partner in partner_obj.browse(cr, uid, [line.partner.id], context=context):
                    if not partner.member_ident:
                        mbr_id = self.pool.get('ir.sequence').get(cr, uid, 'member_ident')
                        partner_obj.write(cr, uid, partner.id, {'member_ident': mbr_id})
                ostate = fetched[0]
                state = 'none'
                if (ostate == 'paid') | (ostate == 'draft'):
                    state = 'waiting'
                elif (ostate == 'done') | (ostate == 'invoiced'):
                    state = 'paid'
                    # XXX: regarder l'équivalent de out_refund pour un pos.order
                    #inv = ord_obj.browse(cr, uid, fetched[1], context=context)
                    #for payment in inv.payment_ids:
                    #    if payment.invoice and payment.invoice.type == 'out_refund':
                    #        state = 'canceled'
                elif ostate == 'cancel':
                    state = 'canceled'
                res[line.id] = state
        return res

    def write(self, cr, uid, ids, vals, context=None):
        res = super(membership_line, self).write(cr, uid, ids, vals, context=context)
        return res

    def create(self, cr, uid, vals, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res = super(membership_line, self).create(cr, uid, vals, context=context)
        return res

    _columns = {
        'pos_order_line': fields.many2one('pos.order.line', 'POS Order line', readonly=True),
        'pos_order_id': fields.related('pos_order_line', 'order_id', type='many2one', relation='pos.order', string='Order', readonly=True),
        'state': fields.function(_state,
                                 string='Membership Status', type='selection',
                                 selection=STATE,
                                 store={
                                 #'account.invoice': (_get_membership_lines_from_account_invoice, ['state'], 10),
                                 # NOTE: déjà géré par membership.membership_line._get_membership_lines
                                 'pos.order': (_get_membership_lines_from_pos_order, ['state', 'partner_id'], 10),
                                 'res.partner': (_get_partners, ['membership_state'], 12),
                                 }, help="""It indicates the membership status.
                                 -Non Member: A member who has not applied for any membership.
                                 -Cancelled Member: A member who has cancelled his membership.
                                 -Old Member: A member whose membership date has expired.
                                 -Waiting Member: A member who has applied for the membership and whose invoice is going to be created.
                                 -Invoiced Member: A member whose invoice has been created.
                                 -Paid Member: A member who has paid the membership amount."""),
        'company_id': fields.related('account_invoice_line', 'invoice_id', 'company_id', type="many2one", relation="res.company", string="Company", readonly=True, store=True)
        # XXX: pos_order_line a aussi un company_id
    }

membership_line()


class Partner(osv.osv):
    '''Partner'''
    _inherit = 'res.partner'

    def _get_partner_id(self, cr, uid, ids, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res_obj = self.pool.get('res.partner')
        data_inv = member_line_obj.browse(cr, uid, ids, context=context)
        list_partner = []
        for data in data_inv:
            list_partner.append(data.partner.id)
        ids2 = list_partner
        while ids2:
            ids2 = res_obj.search(cr, uid, [('associate_member', 'in', ids2)], context=context)
            list_partner += ids2
        return list_partner

    def _get_invoice_partner(self, cr, uid, ids, context=None):
        inv_obj = self.pool.get('account.invoice')
        res_obj = self.pool.get('res.partner')
        data_inv = inv_obj.browse(cr, uid, ids, context=context)
        list_partner = []
        for data in data_inv:
            list_partner.append(data.partner_id.id)
        ids2 = list_partner
        while ids2:
            ids2 = res_obj.search(cr, uid, [('associate_member', 'in', ids2)], context=context)
            list_partner += ids2
        return list_partner

    def _get_order_partner(self, cr, uid, ids, context=None):
        ord_obj = self.pool.get('pos.order')
        res_obj = self.pool.get('res.partner')
        data_ord = ord_obj.browse(cr, uid, ids, context=context)
        list_partner = []
        for data in data_ord:
            list_partner.append(data.partner_id.id)
        ids2 = list_partner
        while ids2:
            ids2 = res_obj.search(cr, uid, [('associate_member', 'in', ids2)], context=context)
            list_partner += ids2
        return list_partner

    def _membership_state(self, cr, uid, ids, name, args, context=None):
        """This Function return Membership State For Given Partner.
        @param self: The object pointer
        @param cr: the current row, from the database cursor,
        @param uid: the current user’s ID for security checks,
        @param ids: List of Partner IDs
        @param name: Field Name
        @param context: A standard dictionary for contextual values
        @param return: Dictionary of Membership state Value
        """
        res = {}
        for id in ids:
            res[id] = 'none'
        today = time.strftime('%Y-%m-%d')
        for id in ids:
            partner_data = self.browse(cr, uid, id, context=context)
            if partner_data.membership_cancel and today > partner_data.membership_cancel:
                res[id] = 'canceled'
                continue
            if partner_data.membership_stop and today > partner_data.membership_stop:
                res[id] = 'old'
                continue
            s = 4
            if partner_data.member_lines:
                for mline in partner_data.member_lines:
                    if mline.date_to >= today:
                        if mline.account_invoice_line and mline.account_invoice_line.invoice_id:
                            istate = mline.account_invoice_line.invoice_id.state
                            if istate == 'paid':
                                s = 0
                                inv = mline.account_invoice_line.invoice_id
                                for payment in inv.payment_ids:
                                    if payment.invoice.type == 'out_refund':
                                        s = 2
                                break
                            elif istate == 'open' and s != 0:
                                s = 1
                            elif istate == 'cancel' and s != 0 and s != 1:
                                s = 2
                            elif (istate == 'draft' or istate == 'proforma') and s != 0 and s != 1:
                                s = 3
                        elif mline.pos_order_line and mline.pos_order_line.order_id:
                            ostate = mline.pos_order_line.order_id.state
                            if ostate == 'paid' and s != 0 and s != 1:
                                s = 3
                            if ostate == 'invoiced':
                                s = 1
                            if ostate == 'done':
                                s = 0
                                # XXX: regarder l'équivalent de out_refund pour un pos.order
                                #inv = mline.pos_order_line.order_id
                                #for payment in inv.payment_ids:
                                #    if payment.invoice.type == 'out_refund':
                                #        s = 2
                                break
                            elif ostate == 'open' and s != 0:
                                # XXX: 1 donne invoiced, c'est pitet pas bon
                                s = 1
                            elif ostate == 'cancel' and s != 0 and s != 1:
                                s = 2
                            elif ostate == 'draft' and s != 0 and s != 1:
                                s = 3
                if s == 4:
                    for mline in partner_data.member_lines:
                        if mline.date_from < today and \
                         mline.date_to < today and \
                         mline.date_from <= mline.date_to and \
                         ((mline.account_invoice_line and mline.account_invoice_line.invoice_id.state) == 'paid' or \
                         (mline.pos_order_line and \
                             (mline.pos_order_line.order_id.state == 'paid' or \
                               mline.pos_order_line.order_id.state == 'done' or \
                               mline.pos_order_line.order_id.state == 'invoiced' ))):
                            s = 5
                        else:
                            s = 6

                if s == 6:
                    # special case when the partner is imported from a file,
                    # in this case is has a valid membership lines but
                    # no payment information associated, consider it valid

                    no_payment_info_at_all = True
                    valid_line = False

                    for mline in partner_data.member_lines:
                        if mline.date_to >= today:
                            valid_line = True

                        if mline.account_invoice_line or mline.pos_order_line:
                            no_payment_info_at_all = False

                    if no_payment_info_at_all and valid_line:
                        s = 0

                if s == 0:
                    res[id] = 'paid'
                elif s == 1:
                    res[id] = 'invoiced'
                elif s == 2:
                    res[id] = 'canceled'
                elif s == 3:
                    res[id] = 'waiting'
                elif s == 5:
                    res[id] = 'old'
                elif s == 6:
                    res[id] = 'none'
            if partner_data.free_member and s != 0:
                res[id] = 'free'
            if partner_data.associate_member:
                res_state = self._membership_state(cr, uid, [partner_data.associate_member.id], name, args, context=context)
                res[id] = res_state[partner_data.associate_member.id]
        return res

    def _membership_date(self, cr, uid, ids, name, args, context=None):
        """Return  date of membership"""
        name = name[0]
        res = {}
        member_line_obj = self.pool.get('membership.membership_line')
        for partner in self.browse(cr, uid, ids, context=context):
            if partner.associate_member:
                 partner_id = partner.associate_member.id
            else:
                 partner_id = partner.id
            res[partner.id] = {
                 'membership_start': False,
                 'membership_stop': False,
                 'membership_cancel': False
            }
            if name == 'membership_start':
                line_id = member_line_obj.search(cr, uid, [('partner', '=', partner_id),('date_cancel','=',False)],
                            limit=1, order='date_from', context=context)
                if line_id:
                        res[partner.id]['membership_start'] = member_line_obj.read(cr, uid, line_id[0],
                                ['date_from'], context=context)['date_from']

            if name == 'membership_stop':
                line_id1 = member_line_obj.search(cr, uid, [('partner', '=', partner_id),('date_cancel','=',False)],
                            limit=1, order='date_to desc', context=context)
                if line_id1:
                      res[partner.id]['membership_stop'] = member_line_obj.read(cr, uid, line_id1[0],
                                ['date_to'], context=context)['date_to']

            if name == 'membership_cancel':
                if partner.membership_state == 'canceled':
                    line_id2 = member_line_obj.search(cr, uid, [('partner', '=', partner.id)], limit=1, order='date_cancel', context=context)
                    if line_id2:
                        res[partner.id]['membership_cancel'] = member_line_obj.read(cr, uid, line_id2[0], ['date_cancel'], context=context)['date_cancel']
        return res

    def _get_partners(self, cr, uid, ids, context=None):
        ids2 = ids
        while ids2:
            ids2 = self.search(cr, uid, [('associate_member', 'in', ids2)], context=context)
            ids += ids2
        return ids

    def __get_membership_state(self, *args, **kwargs):
        return self._membership_state(*args, **kwargs)

    _columns = {
        'membership_state': fields.function(
                    __get_membership_state,
                    string = 'Current Membership Status', type = 'selection',
                    selection = STATE,
                    store = {
                        # NOTE: il est important que la priorité soit plus grande
                        #       que les membership_{start,stop,cancel}
                        #       car _membership_state s'en sert et doit donc les trouver à jour.
                        'account.invoice': (_get_invoice_partner, ['state'], 20),
                        'pos.order': (_get_order_partner, ['state', 'partner_id'], 20),
                        'membership.membership_line': (_get_partner_id, ['state'], 20),
                        'res.partner': (_get_partners, ['free_member', 'membership_state', 'associate_member'], 20)
                    }, help="""It indicates the membership state.
                    -Non Member: A partner who has not applied for any membership.
                    -Cancelled Member: A member who has cancelled his membership.
                    -Old Member: A member whose membership date has expired.
                    -Waiting Member: A member who has applied for the membership and whose invoice is going to be created.
                    -Invoiced Member: A member whose invoice has been created.
                    -Paying member: A member who has paid the membership fee."""),
        'membership_start': fields.function(
                    _membership_date, multi = 'membership_start',
                    string = 'Membership Start Date', type = 'date',
                    store = {
                        'account.invoice': (_get_invoice_partner, ['state'], 10),
                        'pos.order': (_get_order_partner, ['state', 'partner_id'], 10),
                        'membership.membership_line': (_get_partner_id, ['state'], 10),
                        'res.partner': (lambda self, cr, uid, ids, c={}: ids, ['free_member'], 10)
                    }, help="Date from which membership becomes active."),
        'membership_stop': fields.function(
                    _membership_date,
                    string = 'Membership End Date', type='date', multi='membership_stop',
                    store = {
                        'account.invoice': (_get_invoice_partner, ['state'], 10),
                        'pos.order': (_get_order_partner, ['state', 'partner_id'], 10),
                        'membership.membership_line': (_get_partner_id, ['state'], 10),
                        'res.partner': (lambda self, cr, uid, ids, c={}: ids, ['free_member'], 10)
                    }, help="Date until which membership remains active."),
        'membership_cancel': fields.function(
                    _membership_date,
                    string = 'Cancel Membership Date', type='date', multi='membership_cancel',
                    store = {
                        'account.invoice': (_get_invoice_partner, ['state'], 11),
                        'pos.order': (_get_order_partner, ['state', 'partner_id'], 11),
                        'membership.membership_line': (_get_partner_id, ['state'], 10),
                        'res.partner': (lambda self, cr, uid, ids, c={}: ids, ['free_member'], 10)
                    }, help="Date on which membership has been cancelled"),
    }

Partner()

class pos_order(osv.osv):
    _inherit = 'pos.order'

    # XXX: copied from openerp/addons/point_of_sale/point_of_sale.py
    def action_invoice(self, cr, uid, ids, context=None):
        wf_service = netsvc.LocalService("workflow")
        inv_ref = self.pool.get('account.invoice')
        inv_line_ref = self.pool.get('account.invoice.line')
        product_obj = self.pool.get('product.product')
        inv_ids = []

        for order in self.pool.get('pos.order').browse(cr, uid, ids, context=context):
            if order.invoice_id:
                inv_ids.append(order.invoice_id.id)
                continue

            if not order.partner_id:
                raise osv.except_osv(_('Error!'), _('Please provide a partner for the sale.'))

            acc = order.partner_id.property_account_receivable.id
            # XXX: copied from openerp/addons/point_of_sale/point_of_sale.py to fix account.invoice.line creation
            inv = {
                'name': order.name,
                'origin': order.name,
                'account_id': acc,
                'journal_id': order.sale_journal.id or None,
                'type': 'out_invoice',
                'reference': order.name,
                'partner_id': order.partner_id.id,
                'comment': order.note or '',
                'currency_id': order.pricelist_id.currency_id.id, # considering partner's sale pricelist's currency
                'invoice_line': []
            }
            inv.update(inv_ref.onchange_partner_id(cr, uid, [], 'out_invoice', order.partner_id.id)['value'])
            if not inv.get('account_id', None):
                inv['account_id'] = acc
            for line in order.lines:
                inv_line = {
                    #'invoice_id': inv_id,
                    'product_id': line.product_id.id,
                    'quantity': line.qty,
                }
                inv_name = product_obj.name_get(cr, uid, [line.product_id.id], context=context)[0][1]
                inv_line.update(inv_line_ref.product_id_change(cr, uid, [],
                                                               line.product_id.id,
                                                               line.product_id.uom_id.id,
                                                               line.qty, partner_id = order.partner_id.id,
                                                               fposition_id=order.partner_id.property_account_position.id)['value'])
                if line.product_id.description_sale:
                    inv_line['note'] = line.product_id.description_sale
                inv_line['price_unit'] = line.price_unit
                inv_line['discount'] = line.discount
                inv_line['name'] = inv_name
                inv_line['invoice_line_tax_id'] = [(6, 0, [x.id for x in line.product_id.taxes_id] )]
                inv_line_ref.create(cr, uid, inv_line, context=context)
                #inv_line_ref.create(cr, uid, inv_line, context=context)
                inv['invoice_line'].append((0, 0, inv_line))
            inv_id = inv_ref.create(cr, uid, inv, context=context)

            self.write(cr, uid, [order.id], {'invoice_id': inv_id, 'state': 'invoiced'}, context=context)
            inv_ref.confirm_paid(cr, uid, [inv_id], context=context)
            inv_ids.append(inv_id)
            inv_ref.button_reset_taxes(cr, uid, [inv_id], context=context)
            wf_service.trg_validate(uid, 'pos.order', order.id, 'invoice', cr)

        if not inv_ids: return {}

        mod_obj = self.pool.get('ir.model.data')
        res = mod_obj.get_object_reference(cr, uid, 'account', 'invoice_form')
        res_id = res and res[1] or False
        return {
            'name': _('Customer Invoice'),
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [res_id],
            'res_model': 'account.invoice',
            'context': "{'type':'out_invoice'}",
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'current',
            'res_id': inv_ids and inv_ids[0] or False,
        }

    def write(self, cr, uid, ids, vals, context=None):
        pos_order_obj = self.pool.get('pos.order')
        res = super(pos_order, self).write(cr, uid, ids, vals, context=context)
        return res

    def create(self, cr, uid, vals, context=None):
        pos_order_obj = self.pool.get('pos.order')
        res = super(pos_order, self).create(cr, uid, vals, context=context)
        return res

    def onchange_partner_id(self, cr, uid, ids, part=False, context=None):
        if not part:
            return {'value': {}}
        res = super(pos_order, self).onchange_partner_id(cr, uid, ids, part=part, context=context)
        membership_stop = self.pool.get('res.partner').browse(cr, uid, part, context=context).membership_stop
        res['value']['membership_stop'] = membership_stop
        return res

    def _get_membership_stop(self, cr, uid, ids, name, args, context=None):
        """This function return the end date of the partner in pos order."""
        res = {}
        for order in self.browse(cr, uid, ids):
            res[order.id] = order.partner_id.membership_stop
        return res

    _columns = {
        'membership_stop': fields.function(
            _get_membership_stop,
            string='Membership stop',
            type='date',
            help='The end date of the last membership of this partner.',
        ),
    }
pos_order()

class pos_order_line(osv.osv):
    _inherit = 'pos.order.line'

    def write(self, cr, uid, ids, vals, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res = super(pos_order_line, self).write(cr, uid, ids, vals, context=context)
        for line in self.browse(cr, uid, ids, context=context):
            ml_ids = member_line_obj.search(cr, uid, [('pos_order_line', '=', line.id)], context=context)
            if line.product_id and line.product_id.membership:
                member_line_id = member_line_obj.search(cr, uid
                  , [('partner', '=', line.order_id.partner_id.id)
                    ,('pos_order_line', '=', line.id)]
                  , limit=1
                  , context=context)
                for member_line in member_line_obj.browse(cr, uid, member_line_id, context=context):
                  # NOTE: get member_line created in pos_membership.pos_order_line.create
                    date_from = member_line.date_from
                    date_to   = member_line.date_to
                    if line.product_id.membership_date2date:
                        date_from = ('date_from' in context
                          and context.get('date_from')
                          and datetime.strptime(context('date_from'), "%Y-%m-%d")
                          or date.today())
                        date_to   = date_from + relativedelta(months=+12, days=-1) # TODO: parameterize this delta?
                        date_from = date_from.strftime("%Y-%m-%d")
                        date_to   = date_to  .strftime("%Y-%m-%d")
                    member_line_obj.write(cr, uid, member_line.id
                      , {'date_from': date_from
                        ,'date_to'  : date_to
                        }
                      , context=context)
                if line.product_id.membership_grouped:
                    if line.order_id.partner_id.associate_members:
                        associate_member_line_ids = member_line_obj.search(cr, uid
                      , [ ('pos_order_line', '=', line.id)
                        , ('partner', '!=', line.order_id.partner_id.id)
                        ]
                      , context=context)
                    for associate_member_line in member_line_obj.browse(cr, uid, associate_member_line_ids, context=context):
                        member_line_obj.write(cr, uid, associate_member_line.id
                          , {'date_from': date_from
                            ,'date_to'  : date_to
                            }
                          , context=context)
                else:
                    print("DEV: mettre une contrainte pour l'objet\
                          membership.membership_line interdisant les\
                          adhésions groupées reliées à des partenaires\
                          sans membres associés")
            else:
                associate_member_line_ids = member_line_obj.search(cr, uid
                  , [ ('pos_order_line', '=', line.id)
                    , ('partner', '!=', line.order_id.partner_id.id)
                    ]
                  , context=context)
                member_line_obj.unlink(cr, uid, associate_member_line_ids, context=context)
            #Define member ident if it's necessary
            partners = [line.order_id.partner_id]
            if line.order_id.partner_id.associate_members:
                partners.extend(line.order_id.partner_id.associate_members)
            for i in partners:
                if not i.member_ident:
                    mbr_id = self.pool.get('ir.sequence').get(cr, uid, 'member_ident')
                    self.pool.get('res.partner').write(cr, uid, i.id, {'member_ident': mbr_id})
        if line.product_id and not line.product_id.membership and ml_ids:
            # Product line has changed to a non membership product
            member_line_obj.unlink(cr, uid, ml_ids, context=context)
        return res

    def unlink(self, cr, uid, ids, context=None):
        """Remove Membership Line Record for Account Invoice Line
        """
        member_line_obj = self.pool.get('membership.membership_line')
        for id in ids:
            ml_ids = member_line_obj.search(cr, uid, [('pos_order_line', '=', id)], context=context)
            member_line_obj.unlink(cr, uid, ml_ids, context=context)
        return super(pos_order_line, self).unlink(cr, uid, ids, context=context)

    def create(self, cr, uid, vals, context=None):
        member_line_obj = self.pool.get('membership.membership_line')
        res = super(pos_order_line, self).create(cr, uid, vals, context=context)
        line = self.browse(cr, uid, res, context=context)

        ml_ids = member_line_obj.search(cr, uid, [('pos_order_line', '=', line.id)], context=context)
        if line.product_id and line.product_id.membership and not ml_ids:
            date_from = line.product_id.membership_date_from
            date_to = line.product_id.membership_date_to
            if line.order_id.date_order > date_from and line.order_id.date_order < date_to:
                date_from = line.order_id.date_order
            if line.product_id.membership_date2date:
                date_from = ('date_from' in context
                  and context.get('date_from')
                  and datetime.strptime(context.get('date_from'), "%Y-%m-%d")
                  or date.today())
                date_to   = date_from + relativedelta(months=+12, days=-1) # TODO: parameterize this delta?
                date_from = date_from.strftime("%Y-%m-%d")
                date_to   = date_to.strftime("%Y-%m-%d")
            member_line_obj.create(cr, uid, {
                        'partner': line.order_id.partner_id and line.order_id.partner_id.id or False,
                        'membership_id': line.product_id.id,
                        'member_price': line.price_unit,
                        'date': time.strftime('%Y-%m-%d'),
                        'date_from': date_from,
                        'date_to': date_to,
                        'pos_order_line': line.id,
                    }, context=context)
            partners = [line.order_id.partner_id]
            if line.product_id.membership_grouped and line.order_id.partner_id.associate_members:
                partners.extend(line.order_id.partner_id.associate_members)
                #Adding membership lines just for associate partners
                for associate_member in line.order_id.partner_id.associate_members:
                    member_line_obj.create(cr, uid, {
                        'partner': associate_member.id,
                        'membership_id': line.product_id.id,
                        'member_price': line.price_unit,
                        'date': time.strftime('%Y-%m-%d'),
                        'date_from': date_from,
                        'date_to': date_to,
                        'pos_order_line': line.id,
                    }, context=context)
            elif line.product_id.membership_grouped and not line.order_id.partner_id.associate_members:
                raise osv.except_osv(_('Error!!!'), _('You try to order grouped membership product to a partner who hasn\'t associated partners.'))
            #Define member ident if it's necessary
            for i in partners:
                if not i:
                    break
                if not i.member_ident:
                    mbr_id = self.pool.get('ir.sequence').get(cr, uid, 'member_ident')
                    self.pool.get('res.partner').write(cr, uid, i.id, {'member_ident': mbr_id})
        return res

pos_order_line()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
